
-- ff_waterpolo.lua

-----------------------------------------------------------------------------
-- includes
-----------------------------------------------------------------------------
IncludeScript("base_teamplay");
IncludeScript("base_location");

-- set genericbackpack's disallow touch flags for not letting goalies pick up anything
genericbackpack.disallowtouchflags = {AllowFlags.kCivilian}

-----------------------------------------------------------------------------
-- global overrides
-----------------------------------------------------------------------------
POINTS_PER_GOAL = 10
POINTS_PER_GOALIE_RETURN = 5
POINTS_PER_GOALIE_ATTACK = 1
BALL_RETURN_TIME = 10
BALL_THROW_SPEED = 512
GOALIE_SPEED = 2.0
THE_WALL_TIMER_DISABLE = 15
THE_WALL_TIMER_10SECWARN = 5

-- precache (sounds)
function precache()
	PrecacheSound("misc.bizwarn")
	PrecacheSound("misc.bloop")
	PrecacheSound("misc.buzwarn")
	PrecacheSound("misc.dadeda")
	PrecacheSound("misc.deeoo")
	PrecacheSound("misc.doop")
	PrecacheSound("misc.woop")

	PrecacheSound("otherteam.flagstolen")

	-- Unagi Power!  Unagi!
	PrecacheSound("misc.unagi")
end

function startup()

	-- set up team limits (blue & red are unlimited)
	SetPlayerLimit( Team.kBlue, 0 )
	SetPlayerLimit( Team.kRed, 0 )
	SetPlayerLimit( Team.kYellow, -1 )
	SetPlayerLimit( Team.kGreen, -1 )

	SetTeamName( Team.kBlue, "Blue" )
	SetTeamName( Team.kRed, "Red" )

	local team = GetTeam(Team.kBlue)
	team:SetClassLimit( Player.kScout, 0 )
	team:SetClassLimit( Player.kSniper, -1 )
	team:SetClassLimit( Player.kSoldier, 0 )
	team:SetClassLimit( Player.kDemoman, 0 )
	team:SetClassLimit( Player.kMedic, 0 )
	team:SetClassLimit( Player.kHwguy, 0 )
	team:SetClassLimit( Player.kPyro, 0 )
	team:SetClassLimit( Player.kSpy, 0 )
	team:SetClassLimit( Player.kEngineer, 0 )
	team:SetClassLimit( Player.kCivilian, 1 )

	team = GetTeam(Team.kRed)
	team:SetClassLimit( Player.kScout, 0 )
	team:SetClassLimit( Player.kSniper, -1 )
	team:SetClassLimit( Player.kSoldier, 0 )
	team:SetClassLimit( Player.kDemoman, 0 )
	team:SetClassLimit( Player.kMedic, 0 )
	team:SetClassLimit( Player.kHwguy, 0 )
	team:SetClassLimit( Player.kPyro, 0 )
	team:SetClassLimit( Player.kSpy, 0 )
	team:SetClassLimit( Player.kEngineer, 0 )
	team:SetClassLimit( Player.kCivilian, 1 )

	the_wall_reset()

end

function the_wall_reset()
	OutputEvent( "the_wall", "Enable" )
	OutputEvent( "the_wall_laser_blue", "TurnOn" )
	OutputEvent( "the_wall_laser_red", "TurnOn" )
	AddSchedule("the_wall_disable", THE_WALL_TIMER_DISABLE, the_wall_disable )
	AddSchedule("the_wall_10secwarn", THE_WALL_TIMER_10SECWARN, the_wall_10secwarn )

end

function the_wall_disable()
	OutputEvent( "the_wall", "Disable" )
	OutputEvent( "the_wall_laser_blue", "TurnOff" )
	OutputEvent( "the_wall_laser_red", "TurnOff" )
	BroadCastMessage("#FF_ROUND_STARTED")
	BroadCastSound("otherteam.flagstolen")
end

function the_wall_10secwarn()
	BroadCastMessage("#FF_MAP_10SECWARN")
	AddSchedule("the_wall_9secwarn", 1, the_wall_9secwarn )
end

function the_wall_9secwarn()
	BroadCastMessage("#FF_MAP_9SECWARN")
	AddSchedule("the_wall_8secwarn", 1, the_wall_8secwarn )
end

function the_wall_8secwarn()
	BroadCastMessage("#FF_MAP_8SECWARN")
	AddSchedule("the_wall_7secwarn", 1, the_wall_7secwarn )
end

function the_wall_7secwarn()
	BroadCastMessage("#FF_MAP_7SECWARN")
	AddSchedule("the_wall_6secwarn", 1, the_wall_6secwarn )
end

function the_wall_6secwarn()
	BroadCastMessage("#FF_MAP_6SECWARN")
	AddSchedule("the_wall_5secwarn", 1, the_wall_5secwarn )
end

function the_wall_5secwarn()
	BroadCastMessage("#FF_MAP_5SECWARN")
	AddSchedule("the_wall_4secwarn", 1, the_wall_4secwarn )
end

function the_wall_4secwarn()
	BroadCastMessage("#FF_MAP_4SECWARN")
	AddSchedule("the_wall_3secwarn", 1, the_wall_3secwarn )
end

function the_wall_3secwarn()
	BroadCastMessage("#FF_MAP_3SECWARN")
	AddSchedule("the_wall_2secwarn", 1, the_wall_2secwarn )
end

function the_wall_2secwarn()
	BroadCastMessage("#FF_MAP_2SECWARN")
	AddSchedule("the_wall_1secwarn", 1, the_wall_1secwarn )
end

function the_wall_1secwarn()
	BroadCastMessage("#FF_MAP_1SECWARN")
end

-- Give everyone a full resupply, but strip grenades
function player_spawn( player_entity )
	local player = CastToPlayer( player_entity )

	player:AddHealth( 100 )

	player:AddAmmo( Ammo.kNails, 400 )
	player:AddAmmo( Ammo.kShells, 400 )
	player:AddAmmo( Ammo.kRockets, 400 )
	player:AddAmmo( Ammo.kCells, 400 )

	-- god damn right
	player:RemoveAmmo( Ammo.kDetpack, 1 )
	player:RemoveAmmo( Ammo.kManCannon, 1 )
	player:RemoveAmmo( Ammo.kGren1, 4 )
	player:RemoveAmmo( Ammo.kGren2, 4 )

	-- goalies run fast
	if player:GetClass() ~= Player.kCivilian then
		player:AddArmor( 300 )
		player:RemoveEffect( EF.kSpeedlua1 )
	else
		player:RemoveArmor( 300 )
		player:AddEffect( EF.kSpeedlua1, -1, 0, GOALIE_SPEED )
	end
end

function remove_hud_items(ball, player)

	RemoveHudItem( player, ball:GetName() )

	local team = player:GetTeamId()

	if (team == Team.kBlue) then
		RemoveHudItemFromAll( "ball-icon-blue" )
	elseif (team == Team.kRed) then
		RemoveHudItemFromAll( "ball-icon-red" )
	end

end

function player_ondamage( player, damageinfo )
	-- goalies have "UNAGI POWER, UNAGI!"
	local attacker = damageinfo:GetAttacker()
	if IsPlayer( attacker ) then
		attacker = CastToPlayer( attacker )
		if attacker:GetClass() == Player.kCivilian and attacker:GetTeamId() ~= player:GetTeamId() then
			ConsoleToAll( "Goalie, " .. attacker:GetName() .. ", has UNAGI POWER, UNAGI!" )
			BroadCastSound ( "misc.unagi" )
			attacker:AddFortPoints( POINTS_PER_GOALIE_ATTACK * 10, "#FF_FORTPOINTS_GOALIE_ATTACK" )
			damageinfo:ScaleDamage(69)
		end
	end
end




-- ball information
base_ball = info_ff_script:new({
	name = "base ball",
	team = Team.kUnassigned,
	model = "models/items/ball/ball.mdl",
	modelskin = 0,
	tosssound = "Flag.Toss",
	dropnotouchtime = 0,
	capnotouchtime = 2,
	hudicon = "hud_ball",
	hudx = 5,
	hudy = 210,
	hudwidth = 48,
	hudheight = 48,
	hudalign = 1, 
	hudstatusiconbluex = 60,
	hudstatusiconbluey = 5,
	hudstatusiconredx = 60,
	hudstatusiconredy = 5,
	hudstatusiconblue = "hud_ball.vtf",
	hudstatusiconred = "hud_ball.vtf",
	hudstatusiconw = 15,
	hudstatusiconh = 15,
	hudstatusiconbluealign = 2,
	hudstatusiconredalign = 3,
	touchflags = {AllowFlags.kOnlyPlayers,AllowFlags.kBlue, AllowFlags.kRed},
	botgoaltype = Bot.kFlag
})

function base_ball:hasanimation() return false end
function base_ball:usephysics() return true end

function base_ball:touch( touch_entity )
	if IsPlayer( touch_entity ) then

		local player = CastToPlayer( touch_entity )
		if self.notouch[player:GetId()] then return end

		local ball = CastToInfoScript( entity )

		if player:GetClass() ~= Player.kCivilian then
			-- if the player is a spy, then force him to lose his disguise
			player:SetDisguisable( false )
			player:SetCloakable( false )

			ConsoleToAll( player:GetName() .. " has the ball!" )
			SmartSound( player, "misc.bloop", "misc.bloop", "misc.bloop" )
			SmartMessage( player, "#FF_WATERPOLO_YOU_PICKUP", "#FF_WATERPOLO_TEAM_PICKUP", "#FF_WATERPOLO_ENEMY_PICKUP" )
			ball:Pickup( player )

			AddHudIcon( player, self.hudicon, ball:GetName(), self.hudx, self.hudy, self.hudwidth, self.hudheight, self.hudalign )
			local team = player:GetTeamId()
			if (team == Team.kBlue) then
				AddHudIconToAll( self.hudstatusiconblue, "ball-icon-blue", self.hudstatusiconbluex, self.hudstatusiconbluey, self.hudstatusiconw, self.hudstatusiconh, self.hudstatusiconbluealign )
			elseif (team == Team.kRed) then
				AddHudIconToAll( self.hudstatusiconred, "ball-icon-red", self.hudstatusiconredx, self.hudstatusiconredy, self.hudstatusiconw, self.hudstatusiconh, self.hudstatusiconredalign )
			end
		-- goalies return the ball
		else
			if ball:IsDropped() then
				ConsoleToAll( "Goalie, " .. player:GetName() .. ", returned the ball!" )
				SmartSound( player, "misc.deeoo", "misc.deeoo", "misc.deeoo" )
				SmartMessage( player, "#FF_WATERPOLO_YOU_RETURN", "#FF_WATERPOLO_TEAM_GOALIE_RETURN", "#FF_WATERPOLO_ENEMY_GOALIE_RETURN" )
				player:AddFortPoints( POINTS_PER_GOALIE_RETURN * 10, "#FF_FORTPOINTS_GOALIE_RETURN" )
				ball:Return()
			end
		end
	end
end

function base_ball:onownerdie( owner_entity )
	if IsPlayer( owner_entity ) then
		local player = CastToPlayer( owner_entity )
		-- drop the ball
		ConsoleToAll( player:GetName() .. " died and dropped the ball!" )
		local ball = CastToInfoScript( entity )
		ball:Drop(BALL_RETURN_TIME, 0.0)
		remove_hud_items(ball, player)
	end
end

function base_ball:ownerfeign( owner_entity )
	if IsPlayer( owner_entity ) then
		local player = CastToPlayer( owner_entity )
		-- drop the ball
		ConsoleToAll( player:GetName() .. " feigned and dropped the ball!" )
		local ball = CastToInfoScript( entity )
		ball:Drop(BALL_RETURN_TIME, 0.0)
		remove_hud_items(ball, player)
	end
end

function base_ball:dropitemcmd( drop_entity )
	if IsPlayer( drop_entity ) then
		-- throw the ball
		local ball = CastToInfoScript( entity )

		local player = CastToPlayer( drop_entity )
		if not player:IsInUse() then
			ball:Drop(BALL_RETURN_TIME, BALL_THROW_SPEED)
			ConsoleToAll( player:GetName() .. " passed the ball!" )
		else
			-- ball:Drop(BALL_RETURN_TIME, BALL_THROW_SPEED * 2)
			-- speed actually seems to be capped or somehtin' on the C++ side, because watch this ball not go fast at * 100...
			ball:Drop(BALL_RETURN_TIME, BALL_THROW_SPEED * 100)
			ConsoleToAll( player:GetName() .. " passed the ball really far!" )
		end

		remove_hud_items(ball, player)
	
		SmartSound( player, "misc.woop", "misc.woop", "misc.woop" )
	
		-- Make it so the player can't touch the ball for 1 second
		-- (so it can't be thrown and not stick to the player)
		self:addnotouch(player:GetId(), 1)
	end
end

function base_ball:ondrop( owner_entity )
	if IsPlayer( owner_entity ) then
		local player = CastToPlayer( owner_entity )
		-- let the teams know that the flag was dropped
		SmartSound(player, "yourteam.drop", "yourteam.drop", "otherteam.drop")
		SmartMessage(player, "#FF_YOUBALLDROP", "#FF_TEAMBALLDROP", "#FF_ENEMYBALLDROP")
	end
	
	local ball = CastToInfoScript(entity)
	ball:EmitSound(self.tosssound)
end

function base_ball:onloseitem( owner_entity )
	if IsPlayer( owner_entity ) then
		-- let the player that lost the ball put on a disguise
		local player = CastToPlayer( owner_entity )
		player:SetDisguisable(true)
		player:SetCloakable( true )
	
		self:addnotouch(player:GetId(), self.capnotouchtime)
	end
end

function base_ball:spawn()
	self.notouch = { }
	info_ff_script.spawn(self)
end

function base_ball:addnotouch( player_id, duration )
	self.notouch[player_id] = duration
	AddSchedule(self.name .. "-" .. player_id, duration, self.removenotouch, self, player_id)
end

function base_ball.removenotouch(self, player_id)
	self.notouch[player_id] = nil
end

-- For when this object is carried, these offsets are used to place
-- the info_ff_script relative to the players feet
function base_ball:attachoffset()
	-- x = forward/backward
	-- y = left/right
	-- z = up/down
	local offset = Vector( 32, 0, 0 )
	return offset
end

function base_ball:precache()
	PrecacheSound(self.tosssound)
	info_ff_script.precache(self)
end

function base_ball:onreturn()
	ConsoleToAll( "The ball has returned!" )
	BroadCastSound ( "misc.deeoo" )
	BroadCastMessage( "#FF_WATERPOLO_BALL_RETURN" )
end

ball = base_ball:new({
	touchflags = {AllowFlags.kOnlyPlayers,AllowFlags.kBlue, AllowFlags.kRed},
	botgoaltype = Bot.kFlag
})




-- generic goal
base_goal = trigger_ff_script:new({
	health = 100,
	armor = 300,
	grenades = 200,
	nails = 200,
	shells = 200,
	rockets = 200,
	cells = 200,
	detpacks = 0,
	mancannons = 0,
	gren1 = 0,
	gren2 = 0,
	item = "",
	team = 0,
	botgoaltype = Bot.kFlagCap
})

function base_goal:allowed ( allowed_entity )
	if IsPlayer( allowed_entity ) then		
		-- player has to be carrying the item
		local player = CastToPlayer( allowed_entity )
		return player:HasItem( self.item )
	end

	return EVENT_DISALLOWED
end

function base_goal:ontrigger ( trigger_entity )
	if IsPlayer( trigger_entity ) then

		-- player should capture now
		local player = CastToPlayer( trigger_entity )
		local team = GetTeam( self.team )
		team:AddScore( POINTS_PER_CAPTURE )
		ConsoleToAll( team:GetName() .. " team scores!" )

		local ball = GetInfoScriptByName( self.item )
		ball:Return()

		remove_hud_items(ball, player)

		if player:GetTeamId() == self.team then
			ConsoleToAll( player:GetName() .. " scored a goal!" )
			SmartSound( player, "misc.doop", "misc.doop", "misc.dadeda" )
			SmartMessage( player, "#FF_WATERPOLO_YOU_GOAL", "#FF_WATERPOLO_TEAM_GOAL", "#FF_WATERPOLO_ENEMY_GOAL" )
			player:AddFortPoints( POINTS_PER_CAPTURE * 10, "#FF_FORTPOINTS_GOAL" )
		else
			ConsoleToAll( player:GetName() .. " scored an own goal!\nWait, what?!  Kill him like that Columbian a few years ago!  OWN GOAL!  OWN GOAL!" )
			SmartSound( player, "misc.dadeda", "misc.dadeda", "misc.doop" )
			SmartMessage( player, "#FF_WATERPOLO_YOU_OWN_GOAL", "#FF_WATERPOLO_TEAM_OWN_GOAL", "#FF_WATERPOLO_ENEMY_OWN_GOAL" )
			player:AddFortPoints( -POINTS_PER_CAPTURE * 10, "#FF_FORTPOINTS_OWN_GOAL" )
		end

		ApplyToAll({ AT.kRespawnPlayers, AT.kReloadClips })

		the_wall_reset()
	end
end

-- on blue's side of map, but points go to red
blue_goal = base_goal:new({ item = "ball", team = Team.kRed })

-- on red's side of map, but points go to blue
red_goal = base_goal:new({ item = "ball", team = Team.kBlue })




-- returns the item
base_item_returner = trigger_ff_script:new({
	item = "",
	message = ""
})

function base_item_returner:allowed ( allowed_entity )

	if IsPlayer( allowed_entity ) then
		-- player has to be carrying the item
		local player = CastToPlayer( allowed_entity )
		return player:HasItem( self.item )
	end

	return EVENT_ALLOWED

end

function base_item_returner:ontrigger ( trigger_entity )

	if IsPlayer( trigger_entity ) then
		local player = CastToPlayer( trigger_entity )
		-- return the item
		local item = GetInfoScriptByName( self.item )
		item:Return()
		remove_hud_items(item, player)
		ConsoleToAll( player:GetName() .. self.message )
		SmartSound( player, "misc.deeoo", "misc.deeoo", "misc.deeoo" )
		SmartMessage( player, "#FF_WATERPOLO_YOU_BOUNDS", "#FF_WATERPOLO_TEAM_BOUNDS", "#FF_WATERPOLO_ENEMY_BOUNDS" )
	end

end

-- ball stripper
ball_stripper = base_item_returner:new({ item = "ball", message = " took the ball out of bounds!" })

-- spawn ammo packs
waterpolo_pack_spawn_blue = blue_ammobackpack
waterpolo_pack_spawn_red = red_ammobackpack

-- unique waterpolo spawns (goalies spawn on top of the goal)
bluespawn = { validspawn = function(self,player) return player:GetTeamId() == Team.kBlue and player:GetClass() ~= Player.kCivilian end }
redspawn = { validspawn = function(self,player) return player:GetTeamId() == Team.kRed and player:GetClass() ~= Player.kCivilian end }
bluespawn_goalie = { validspawn = function(self,player) return player:GetTeamId() == Team.kBlue and player:GetClass() == Player.kCivilian end }
redspawn_goalie = { validspawn = function(self,player) return player:GetTeamId() == Team.kRed and player:GetClass() == Player.kCivilian end }


-----------------------------------------------------------------------------
-- unique waterpolo locations
-----------------------------------------------------------------------------
location_ballsonia = location_info:new({ text = "Ballsonia", team = Team.kUnassigned })
location_midfield = location_info:new({ text = "Midfield", team = Team.kUnassigned })
location_blue_side = location_info:new({ text = "Blue Side", team = Team.kBlue })
location_red_side = location_info:new({ text = "Red Side", team = Team.kRed })
location_blue_goal = location_info:new({ text = "Blue Goal", team = Team.kBlue })
location_red_goal = location_info:new({ text = "Red Goal", team = Team.kRed })
location_upper_deck = location_info:new({ text = "Upper Deck", team = Team.kUnassigned })
location_blue_upper_deck = location_info:new({ text = "Blue Upper Deck", team = Team.kBlue })
location_red_upper_deck = location_info:new({ text = "Red Upper Deck", team = Team.kRed })
location_nose_bleeds = location_info:new({ text = "Nose Bleeds", team = Team.kUnassigned })
location_blue_nose_bleeds = location_info:new({ text = "Blue Nose Bleeds", team = Team.kBlue })
location_red_nose_bleeds = location_info:new({ text = "Red Nose Bleeds", team = Team.kRed })
location_rainy_sky = location_info:new({ text = "Rainy Sky", team = Team.kUnassigned })
location_death_hills = location_info:new({ text = "Death Hills", team = Team.kUnassigned })
location_sideroom1 = location_info:new({ text = "Side Room 1", team = Team.kUnassigned })
location_sideroom2 = location_info:new({ text = "Side Room 2", team = Team.kUnassigned })
